from crea_app.models import CustomUser
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status, permissions
from rest_framework.exceptions import ValidationError, NotFound
from django.db.models import Count, Q
from django.shortcuts import get_object_or_404
from ast import literal_eval
from django.http import JsonResponse
import json
from django.utils.timezone import now
import random
from userprofile_app.models import Profile
from .models import Forum, ForumInvitation, ForumMembership, ForumQuestion, Like, Comment, Notification  
from .serializers import *
from django.shortcuts import get_object_or_404
from rest_framework.exceptions import ValidationError
from rest_framework import status
from django.db.models import Count
from django.db.models import Count, Exists, OuterRef

class CreateForumView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request):
        data = request.data.copy()
        data['user'] = request.user.id

        # Handle categories as a string and split into a list
        categories_data = data.get('categories', None)
        
        if categories_data:
            if isinstance(categories_data, str):
                try:
                    # Split string by commas to create a list of category IDs
                    categories_data = [int(cat_id.strip()) for cat_id in categories_data.split(",")]
                except ValueError:
                    raise ValidationError({"categories": "Categories must be a comma-separated string of integers."})
            else:
                raise ValidationError({"categories": "Expected categories to be a string."})

        # Validate categories against Industry model
        valid_categories = Industry.objects.filter(id__in=categories_data)
        if len(valid_categories) != len(categories_data):
            raise ValidationError({"categories": "Some of the categories are invalid."})
        
        invite_user_ids = data.get('invite_user_ids', [])
        if isinstance(invite_user_ids, str):
            try:
                # Convert comma-separated string to a list of integers
                invite_user_ids = [int(user_id.strip()) for user_id in invite_user_ids.split(",")]
            except ValueError:
                raise ValidationError({"invite_user_ids": "Invite user IDs must be a comma-separated string of integers."})
        elif not isinstance(invite_user_ids, list):
            raise ValidationError({"invite_user_ids": "Expected invite_user_ids to be a list or comma-separated string."})

        # Remove categories from data before passing to the serializer
        data.pop('categories', None)
        data.pop('invite_user_ids', None)

        # Save the forum
        serializer = ForumSerializer(data=data)
        if serializer.is_valid():
            forum = serializer.save()

            # Set categories on the forum
            forum.categories.set(categories_data)

            forum.invite_user_ids = invite_user_ids
            forum.save()


            # Create notification for the user
            Notification.objects.create(
                user=request.user,  # Ensure this is a CustomUser instance
                message=f"You created the forum '{forum.title}'."
            )

            # Handle invited users
            if forum.permission_to_join == 'private':
                for user_id in forum.invite_user_ids:
                    invited_user = get_object_or_404(CustomUser, id=user_id)  # Ensure invited_user is an instance
                    ForumInvitation.objects.get_or_create(
                        forum=forum,
                        invited_user=invited_user,
                        invited_by=forum.user,
                        defaults={"status": "pending"}
                    )
                    Notification.objects.create(
                        user=invited_user,  # Ensure this is a CustomUser instance
                        message=f"You were invited to join the forum '{forum.title}'."
                    )
            if forum.permission_to_join == 'public':
                # Automatically add all invited users to forum membership
                for user_id in forum.invite_user_ids:
                    user = get_object_or_404(CustomUser, id=user_id)
                    ForumMembership.objects.get_or_create(
                        forum=forum,
                        user=user,
                        defaults={'is_active': True}
                    )

            # Add the creator to the forum membership
            ForumMembership.objects.get_or_create(
                forum=forum,
                user=forum.user
            )

            return Response(
                {"success": True, "message": "Forum created successfully."},
                status=status.HTTP_201_CREATED,
            )

        raise ValidationError(serializer.errors)

# class CreateForumView(APIView):
#     permission_classes = [permissions.IsAuthenticated]

#     def post(self, request):
#         data = request.data.copy()
#         data['user'] = request.user.id
#         categories_data = data.get('categories', None)
        
#         if categories_data is not None:

#             if isinstance(categories_data, str):
#                 try:

#                     categories_data = json.loads(categories_data)
#                 except json.JSONDecodeError:
#                     raise ValidationError({"categories": "Expected a list of category IDs."})
#             data['categories'] = categories_data
#         serializer = ForumSerializer(data=data)

#         if serializer.is_valid():
#             forum = serializer.save()

#             Notification.objects.create(
#                 user=request.user,
#                 message=f"You created the forum '{forum.title}'."
#             )

#             for user_id in forum.invite_user_ids:
#                 ForumInvitation.objects.get_or_create(
#                     forum=forum,
#                     invited_user_id=user_id,
#                     invited_by=forum.user,
#                     defaults={"status": "pending"}
#                 )
#                 Notification.objects.create(
#                     user=user_id,
#                     message=f"You were invited to join the forum '{forum.title}'."
#                 )

#             ForumMembership.objects.get_or_create(
#                 forum = forum,
#                 user = forum.user
#             )

#             return Response(
#                 {"success": True, "message": "Forum created successfully."},
#                 status=status.HTTP_201_CREATED,
#             )
#         raise ValidationError(serializer.errors)

class EditForumView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def put(self, request, forum_id):
        try:
            forum = Forum.objects.get(id=forum_id, user=request.user, is_deleted=False)
        except Forum.DoesNotExist:
            return Response({"success": False, "message": "Forum not found or unauthorized action."}, status=404)

        data = request.data.copy()
        serializer = ForumSerializer(forum, data=data, partial=True)  # Allow partial updates
        if serializer.is_valid():
            forum = serializer.save()
            return Response({"success": True, "message": "Forum updated successfully."}, status=status.HTTP_200_OK)

        return Response({"success": False, "message": "Invalid data.", "errors": serializer.errors}, status=400)

class DeleteForumView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def delete(self, request, forum_id):
        try:
            forum = Forum.objects.get(id=forum_id, user=request.user, is_deleted=False)
        except Forum.DoesNotExist:
            return Response({"success": False, "message": "Forum not found or unauthorized action."}, status=404)

        # Soft delete the forum
        forum.is_deleted = True
        forum.save()

        members = ForumMembership.objects.filter(forum=forum, is_active=True)
        for member in members:
            Notification.objects.create(
                user=member.user,
                message=f"The forum '{forum.title}' has been deleted by {request.user.username}."
            )

        return Response({"success": True, "message": "Forum deleted successfully."}, status=status.HTTP_200_OK)



# class InviteFriendView(APIView):
#     permission_classes = [permissions.IsAuthenticated]

#     def post(self, request):
#         data = request.data.copy()
#         if isinstance(data.get('userIds'), str):
#             try:
#                 data['userIds'] = literal_eval(data['userIds'])
#             except (ValueError, SyntaxError):
#                 return Response(
#                     {"success": False, "message": "Invalid format for userIds. Must be a list of integers."},
#                     status=400
#                 )
#         serializer = ForumInviteSerializer(data=data)

#         if serializer.is_valid():
#             forum_id = serializer.validated_data['forum_id']
#             user_ids = serializer.validated_data['userIds'][0]
#             print(user_ids)
#             invited_by = request.user

#             try:
#                 forum = Forum.objects.get(id=forum_id, user=invited_by)
#             except Forum.DoesNotExist:
#                 return Response({"success": False, "message": "Forum not found or unauthorized action."}, status=403)
            
#             if forum.user != invited_by:
#                 return Response({"success": False, "message": "Only forum creator can invite users."}, status=403)

#             try:
#                 already_invited_users = []
#                 newly_invited_users = []
#                 for user_id in user_ids:
#                     existing_invitation = ForumInvitation.objects.filter(
#                         forum=forum,
#                         invited_user_id=user_id,
#                     ).exclude(status="declined").exists()

#                     if existing_invitation:
#                         already_invited_users.append(user_id)
#                     else:
#                         ForumInvitation.objects.create(
#                             forum=forum,
#                             invited_user_id=user_id,
#                             invited_by=invited_by,
#                             status="pending"
#                         )
#                         Notification.objects.create(
#                             user=user_id,
#                             message=f"You have been invited to the forum '{forum.title}'."
#                         )
#                         newly_invited_users.append(user_id)

#                 # Construct response
#                 response_data = {"success": True, "message": "Users invited successfully."}
#                 if not newly_invited_users:
#                     response_data['message'] = 'Already invited'
#                 return Response(response_data)
#             except Exception as e:
#                 print(e)
#                 return Response({"success": False, "message": "Something went wrong while creating the invitation"}, status=403)

#         raise ValidationError(serializer.errors)
import json

class InviteFriendView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request):
        data = request.data
        user_ids = data.get('userIds')
        forum_id = data.get('forum_id')

        if isinstance(user_ids, str):
            try:
                user_ids = json.loads(user_ids)
            except json.JSONDecodeError:
                return Response(
                    {"success": False, "message": "Invalid format for userIds. Must be a valid JSON array of integers."},
                    status=400
                )

        # Validate that user_ids is a list of integers
        if not isinstance(user_ids, list) or not all(isinstance(i, int) for i in user_ids):
            return Response(
                {"success": False, "message": "Invalid format for userIds. Must be a list of integers."},
                status=400
            )

        # Fetch the forum and validate permissions
        try:
            forum = Forum.objects.get(id=forum_id, user=request.user)
        except Forum.DoesNotExist:
            return Response({"success": False, "message": "Forum not found or unauthorized action."}, status=403)

        already_invited_users = []
        newly_invited_users = []

        # Process invitations
        for user_id in user_ids:
            try:
                if ForumInvitation.objects.filter(
                    forum=forum, invited_user_id=user_id
                ).exclude(status="declined").exists():
                    already_invited_users.append(user_id)
                else:
                    ForumInvitation.objects.create(
                        forum=forum,
                        invited_user_id=user_id,
                        invited_by=request.user,
                        status="pending"
                    )
                    Notification.objects.create(
                        user_id=user_id,
                        message=f"You have been invited to the forum '{forum.title}'."
                    )
                    newly_invited_users.append(user_id)
            except Exception as e:
                print(f"Error inviting user {user_id}: {e}")

        response_data = {
            "success": True,
            "message": "Users invited successfully.",
            "already_invited": already_invited_users,
            "newly_invited": newly_invited_users
        }
        if not newly_invited_users:
            response_data["message"] = "All users were already invited."
        return Response(response_data, status=200)


class AcceptInvitationView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request):
        serializer = ForumActionSerializer(data=request.data)
        if serializer.is_valid():
            user_id = serializer.validated_data['userId']
            forum_id = serializer.validated_data['forum_id']

            try:
                invitation = ForumInvitation.objects.get(
                    forum_id=forum_id, invited_user_id=user_id, status="pending"
                )
            except ForumInvitation.DoesNotExist:
                return Response({"success": False, "message": "Invitation not found or already processed."}, status=404)

            invitation.status = "accepted"
            invitation.save()

            ForumMembership.objects.get_or_create(user_id=user_id, forum_id=forum_id, defaults={"is_active": True})

            Notification.objects.create(
            user=invitation.invited_by,
            message=f"{invitation.invited_user.username} has accepted the invitation to join '{invitation.forum.title}'."
            )

            # Notify the user
            Notification.objects.create(
            user=invitation.invited_user,
            message=f"You have successfully joined the forum '{invitation.forum.title}'."
            )

            return Response({"success": True, "message": "Invitation accepted. Welcome to the forum!"})
        return Response(serializer.errors, status=400)

class DeclineInvitationView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request):
        serializer = ForumActionSerializer(data=request.data)
        if serializer.is_valid():
            user_id = serializer.validated_data['userId']
            forum_id = serializer.validated_data['forum_id']

            try:
                invitation = ForumInvitation.objects.get(
                    forum_id=forum_id, invited_user_id=user_id, status="pending"
                )
            except ForumInvitation.DoesNotExist:
                return Response({"success": False, "message": "Invitation not found or already processed."}, status=404)

            invitation.status = "declined"
            invitation.save()

            Notification.objects.create(
            user=invitation.invited_by,
            message=f"{invitation.invited_user.username} has declined the invitation to join '{invitation.forum.title}'."
            )

            return Response({"success": True, "message": "Invitation declined."})
        return Response(serializer.errors, status=400)

class ExitForumView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request):
        serializer = ForumActionSerializer(data=request.data)
        if serializer.is_valid():
            user_id = serializer.validated_data['userId']
            forum_id = serializer.validated_data['forum_id']

            try:
                membership = ForumMembership.objects.get(user_id=user_id, forum_id=forum_id, is_active=True)
            except ForumMembership.DoesNotExist:
                return Response({"success": False, "message": "Membership not found."}, status=404)

            membership.is_active = False
            membership.save()

            Notification.objects.create(
            user=membership.forum.user,
            message=f"{membership.user.username} has exited the forum '{membership.forum.title}'."
            )

            return Response({"success": True, "message": "You have exited the forum."})
        return Response(serializer.errors, status=400)

class RemoveUserView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def delete(self, request):
        serializer = ForumRemoveUserSerializer(data=request.data)
        if serializer.is_valid():
            removed_by = serializer.validated_data['removedBy']
            forum_id = serializer.validated_data['forum_id']
            user_id = serializer.validated_data['userId']

            try:
                forum = Forum.objects.get(id=forum_id, user_id=removed_by)
            except Forum.DoesNotExist:
                return Response({"success": False, "message": "Only the forum owner can remove users."}, status=403)

            membership = ForumMembership.objects.filter(user_id=user_id, forum_id=forum_id, is_active=True).first()
            if not membership:
                return Response({"success": False, "message": "User not found in the forum."}, status=404)

            membership.is_active = False
            membership.save()

            Notification.objects.create(
                user=membership.user,
                message=f"You were removed from the forum '{forum.title}' by {forum.user.username}."
            )

            # Notify the forum creator
            Notification.objects.create(
                user=forum.user,
                message=f"{membership.user.username} was removed from the forum '{forum.title}'."
            )

            return Response({"success": True, "message": "User removed from the forum."})
        return Response(serializer.errors, status=400)


class ForumDetailsView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def get(self, request):
        forum_id = request.query_params.get("forum_id") 
        user_id = request.query_params.get("userId") 

        if not forum_id or not user_id:
            return Response(
                {"success": False, "message": "forum_id and userId are required."},
                status=status.HTTP_400_BAD_REQUEST,
            )

        
        forum = get_object_or_404(Forum, id=forum_id)

        owner_status = forum.user.id == int(user_id)

        members_count = ForumMembership.objects.filter(forum=forum, is_active=True).count()

        creator_profile = get_object_or_404(Profile, user = forum.user)
        profile_image_url = (
            creator_profile.profile_image.url if creator_profile and creator_profile.profile_image else None
        )

        category_names = list(forum.categories.values_list('name', flat=True)) if forum.categories.exists() else ["Film"]
        forum_questions = ForumQuestion.objects.filter(forum= forum_id, is_deleted=False).order_by("-created_at")
        questions_data= []
        for question in forum_questions:
            # Calculate total likes and user-specific like status
            total_likes = Like.objects.filter(question=question).count()
            user_liked = Like.objects.filter(question=question, user_id=user_id).exists()

            # Check if the question has comments
            total_comments = Comment.objects.filter(question=question).count()
            

            questions_data.append({
                "questionId": question.id,
                "title": question.question,
                "content": question.content,
                "tags": question.tags,
                "totalLikes": total_likes,
                "likedByUser": user_liked,
                "totalComments": total_comments,
                "createdBy": {
                    "username": question.user.username,
                    "profileImage": question.user.profile.profile_image.url if question.user.profile.profile_image else None,
                    "full_name": question.user.full_name
                },
                "createdDate": question.created_at,
            })
        
        # Construct response
        response_data = {
            "forumId": forum.id,
            "title": forum.title,
            "owner_status": str(owner_status).lower(),
            "description": forum.short_description,
            "category": category_names,
            "coverImage": forum.cover_image.url if forum.cover_image else None,
            "createdBy": {
                "username": forum.user.username,
                "profileImage": profile_image_url,
                "full_name": forum.user.full_name
            },
            "createdDate": forum.created_at,
            "tags": forum.hashtags,
            "membersCount": members_count,
            "permissions": forum.permission_to_join.capitalize(),
            "questions": questions_data,
        }

        return Response(response_data, status=status.HTTP_200_OK)

class DiscussionMembersView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def get(self, request):
        forum_id = request.query_params.get("forum_id")
        user_id = request.query_params.get("userId")

        if not forum_id or not user_id:
            return Response(
                {"success": False, "message": "forum_id and user_id are required."},
                status=status.HTTP_400_BAD_REQUEST,
            )

        forum = get_object_or_404(Forum, id=forum_id)

        if not ForumMembership.objects.filter(forum=forum, user_id=user_id, is_active=True).exists():
            return Response(
                {"success": False, "message": "Unauthorized access to the forum."},
                status=status.HTTP_403_FORBIDDEN,
            )

        active_members = ForumMembership.objects.filter(forum=forum, is_active=True)
        pending_invitations = ForumInvitation.objects.filter(forum=forum, status="pending")

        members_data = []

        for membership in active_members:
            user_profile = get_object_or_404(Profile, user = membership.user)
            profile_image_url = (
                user_profile.profile_image.url if user_profile and user_profile.profile_image else None
            )
            members_data.append({
                "userId": membership.user.id,
                "username": membership.user.username,
                "profileImage": profile_image_url,
                "answersCount": random.randint(1, 20),  # Dummy data for answers count
                "lastActiveDate": "2024-11-28",  # Dummy data for last active date
                "joined_status": "Joined" 
            })

        for invitation in pending_invitations:
            user_profile = get_object_or_404(Profile, user = invitation.invited_user)
            profile_image_url = (
                user_profile.profile_image.url if user_profile and user_profile.profile_image else None
            )
            members_data.append({
                "userId": f"U{invitation.invited_user.id}",
                "username": invitation.invited_user.username,
                "profileImage": profile_image_url,
                "answersCount": random.randint(1, 20),  # Dummy data for answers count
                "lastActiveDate": "2024-11-28",  # Dummy data for last active date
                "joined_status": "Waiting for accept" 
            })

        if not members_data:
            return Response(
                {"success": False, "message": "Members not found."},
                status=status.HTTP_404_NOT_FOUND,
            )


        response_data = {
            "forumId": forum.id,
            "created_by" : forum.user.username,
            "forumTitle": forum.title,
            "membersCount": len(members_data),
            "discussionMembers": members_data,
        }

        return Response(response_data, status=status.HTTP_200_OK)

class CreateQuestionView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request):
        data = request.data.copy()
        data['user'] = request.user.id
        forum_id = data['forum_id']
        if not forum_id:
            return Response(
                {"success": False, "message": "forum_id is required."},
                status=status.HTTP_404_NOT_FOUND,
            )
        data['forum'] = forum_id

        serializer = ForumQuestionSerializer(data=data)
        if serializer.is_valid():
            question = serializer.save()

            members = ForumMembership.objects.filter(forum_id=forum_id, is_active=True).exclude(user=request.user)
            for member in members:
                Notification.objects.create(
                    user=member.user,
                    message=f"A new question '{question.question}' has been posted in the forum '{question.forum.title}' by {request.user.username}."
                )

            return Response({
                "success": True,
                "message": "Question created successfully.",
                "question_id": question.id
            }, status=status.HTTP_201_CREATED)

        raise ValidationError(serializer.errors)

class DeleteQuestionView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def delete(self, request, question_id):
        user = request.user

        try:
            question = ForumQuestion.objects.get(id=question_id, is_deleted = False)
        except ForumQuestion.DoesNotExist:
            return Response({
                "success": False,
                "message": "Question not found"
            }, status=status.HTTP_404_NOT_FOUND)

        # Ensure only the author or forum admin can delete the question
        if question.user != user or question.forum.user != user:
            return Response({
                "success": False,
                "message": "You do not have permission to delete this question."
            }, status=status.HTTP_403_FORBIDDEN)

        question.is_deleted = True
        question.save()

        members = ForumMembership.objects.filter(forum=question.forum, is_active=True)
        for member in members:
            Notification.objects.create(
                user=member.user,
                message=f"The question '{question.title}' in the forum '{question.forum.title}' has been deleted by {request.user.username}."
            )

        return Response({
            "success": False,
            "message": "Question deleted successfully."
        }, status=status.HTTP_200_OK)

class UpdateQuestionView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def put(self, request, question_id):
        user = request.user
        data = request.data

        try:
            question = ForumQuestion.objects.get(id=question_id)
        except ForumQuestion.DoesNotExist:
            return Response({
                "success": False,
                "message": "Question not found."
            }, status=status.HTTP_404_NOT_FOUND)

        # Ensure only the author of the question can update it
        if question.user != user:
            return Response({
                "success": False,
                "message": "You do not have permission to update this question."
            }, status=status.HTTP_403_FORBIDDEN)

        # Validate and update the question
        serializer = ForumQuestionSerializer(question, data=data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response({
                "success": True,
                "message": "Question updated successfully."
            }, status=status.HTTP_200_OK)

        raise ValidationError(serializer.errors)

class AddCommentView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request, question_id):
        try:
            question = ForumQuestion.objects.get(id=question_id)
        except ForumQuestion.DoesNotExist:
            return Response({"success": False, "message": "Question not found."}, status=status.HTTP_404_NOT_FOUND)

        data = request.data.copy()
        data['question'] = question.id
        data['user'] = request.user.id 
        
        serializer = CommentSerializer(data=data)
        if serializer.is_valid():
            serializer.save()

            if question.user != request.user:
                Notification.objects.create(
                    user=question.user,
                    message=f"A new comment has been added to your question in the forum '{question.forum.title}' by {request.user.username}."
                )
            return Response({"success": True, "message": "Comment added successfully."}, status=status.HTTP_201_CREATED)
        
        return Response({"success": False, "message": serializer.errors}, status=status.HTTP_400_BAD_REQUEST)

class LikeQuestionView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request, question_id):
        user = request.user

        try:
            question = ForumQuestion.objects.get(id=question_id)
        except ForumQuestion.DoesNotExist:
            return Response({"success": False, "message": "Question not found."}, status=status.HTTP_404_NOT_FOUND)

        # Check if the user has already liked the question
        like, created = Like.objects.get_or_create(user=user, question=question)

        if not created:
            return Response({
                "success": False,
                "message": "You have already liked this question."
            }, status=status.HTTP_400_BAD_REQUEST)

        if question.user != request.user:
            Notification.objects.create(
                user=question.user,
                message=f"{request.user.username} liked your question in the forum '{question.forum.title}'."
            )

        return Response({
            "success": True,
            "message": "Liked successfully."
        }, status=status.HTTP_201_CREATED)


class ShareQuestionView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request, question_id):

        question = get_object_or_404(ForumQuestion, id=question_id)


        base_url = request.build_absolute_uri('/')[:-1]
        shareable_url = f"{base_url}/api/forum/questions/{question_id}"

        return Response({
            "success": True,
            "url_question": shareable_url,
            "message": "Question link shared successfully."
        }, status=status.HTTP_200_OK)


class GetQuestionDetailView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def get(self, request, question_id):
        try:
            # Fetch the question
            question = ForumQuestion.objects.get(id=question_id, is_deleted=False)

            # Serialize the question details
            question_data = {
                "question_id": question.id,
                "forum_id": question.forum.id,
                "title": question.question,
                "content": question.content,
                "tags": question.tags,
                "created_at": question.created_at,
                "likes_count": question.question_likes.count(),
                "comments": [
                    {
                        "comment_id": comment.id,
                        "user_id": comment.user.id,
                        "comment": comment.comment,
                        "created_at": comment.created_at
                    }
                    for comment in question.question_comments.filter(is_deleted=False)
                ]
            }

            return Response(
                {"success": True, "data": question_data},
                status=status.HTTP_200_OK
            )
        except ForumQuestion.DoesNotExist:
            return Response(
                {"success": False, "message": "Question not found or deleted."},
                status=status.HTTP_404_NOT_FOUND
            )

class ForumMemberListView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def get(self, request, forum_id):
        user = request.user

        try:
            forum = Forum.objects.get(id=forum_id)
        except Forum.DoesNotExist:
            return Response({"success": True, "message": "Forum not found or unauthorized access."}, status=404)

        if not ForumMembership.objects.filter(forum=forum, user=user).exists():
            return Response({"success": False, "message": "Unauthorized access to this forum."}, status=403)


        members = ForumMembership.objects.filter(forum=forum).select_related('user')

        if not members.exists():
            return Response({"success": False, "message": "No members found for this forum."}, status=404)

        serializer = ForumMemberSerializer(members, many=True)
        return Response({
            "success": True,
            'data':{
                "forum_id": forum.id,
                "forum_name": forum.title,
                "members": serializer.data
            }        
        })

class ForumListView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def get(self, request):
        user = request.user
        category_ids = request.query_params.getlist("category_ids[]")
        scope = request.query_params.get("scope")
        keyword = request.query_params.get("keyword", '')


        forums = Forum.objects.filter(permission_to_join="public", is_deleted=False)


        if category_ids:
            forums = forums.filter(category_id__in=category_ids)


        if scope == "created":
            forums = forums.filter(user=user)
        elif scope == "others":
            forums = forums.exclude(user=user).distinct()
        
        if keyword:
            forums = forums.filter(
                Q(title__icontains=keyword) | Q(short_description__icontains=keyword)
            )

        serializer = ForumViewSerializer(forums, many=True)

        if not forums.exists():
            return Response(
                {"status": "error", "message": "No forums found for the given criteria."},
                status=404
            )

        return Response({"status": "success", "forums": serializer.data})
    

class ForumListWithMetricsAPIView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def get(self, request, forum_id):

        try:
            forum = Forum.objects.get(id=forum_id, is_deleted=False)
        except Forum.DoesNotExist:
            raise NotFound({"status": "error", "message": "Forum not found"})


        total_answers = forum.questions.filter(is_deleted=False).count()
        total_users = forum.questions.filter(is_deleted=False).values('user').distinct().count()

        questions = forum.questions.filter(is_deleted=False).annotate(
            comment_count=Count('question_comments', filter=Q(question_comments__is_deleted=False)),
            like_count=Count('question_likes', filter=Q(question_likes__is_deleted=False)),
            interaction_count=Count('question_comments', filter=Q(question_comments__is_deleted=False))
            + Count('question_likes', filter=Q(question_likes__is_deleted=False))
        ).order_by('-interaction_count')[:10] 


        question_data = [
            {
                "question_id": question.id,
                "question": question.question,
                "content": question.content,
                "tags": question.tags,
                "created_at": question.created_at,
                "comment_count": question.comment_count,
                "like_count": question.like_count,
                "interaction_count": question.interaction_count,
            }
            for question in questions
        ]

        forum_data = {
            "forum_id": forum.id,
            "forum_title": forum.title,
            "short_description": forum.short_description,
            "category": forum.categories.name if forum.categories else None,
            "permission_to_join": forum.permission_to_join,
            "total_answers": total_answers,
            "total_users": total_users,
            "created_at": forum.created_at,
            "cover_image": forum.cover_image.url if forum.cover_image else None,
            "top_questions": question_data,
        }

        return Response({"status": "success", "forum": forum_data})

class TopForumsView(APIView):
    def get(self, request):
        # Query forums, annotate with member count and question count, and order by member count
        forums = (
            Forum.objects.annotate(
                members_count=Count('memberships', filter=Q(memberships__is_active=True)),
                questions_count=Count('questions', filter=Q(questions__is_deleted=False))
            )
            .order_by('-members_count')[:20]  # Limit to top 20 forums
        )

        # Serialize response data
        data = [
            {
                "forumId": forum.id,
                "title": forum.title,
                "user_id":forum.user_id,
                "full_name":forum.user.full_name,
                "shortDescription": forum.short_description,
                "membersCount": forum.members_count,
                "questionsCount": forum.questions_count,
                "createdBy": forum.user.username,
                "createdDate": forum.created_at.strftime("%Y-%m-%d"),
                "category": [category.name for category in forum.categories.all()],
                "coverImage": forum.cover_image.url if forum.cover_image else None,
            }
            for forum in forums
        ]

        return Response({"success": True, "data": data}, status=status.HTTP_200_OK)
    
class JoinPublicForumView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request):
        forum_id = request.data.get("forum_id")

        try:
            forum = Forum.objects.get(id=forum_id)
        except Forum.DoesNotExist:
            return Response(
                {"success": False, "message": "Forum not found."},
                status=status.HTTP_404_NOT_FOUND
            )

        if forum.permission_to_join != "public":
            return Response(
                {"success": False, "message": "You cannot join this forum. It is not public."},
                status=status.HTTP_403_FORBIDDEN
            )

        user = request.user
        if ForumMembership.objects.filter(forum=forum, user=user).exists():
            return Response(
                {"success": False, "message": "You are already a member of this forum."},
                status=status.HTTP_400_BAD_REQUEST
            )

        # Add the user to the forum membership
        ForumMembership.objects.create(forum=forum, user=user)

        return Response(
            {"success": True, "message": "You have successfully joined the forum."},
            status=status.HTTP_200_OK
        )

class ForumsCreatedByMeView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def get(self, request):
        user = request.user
        keyword = request.query_params.get("keyword", '')

        forums = Forum.objects.filter(
            user=user, 
            permission_to_join="public", 
            is_deleted=False
        )

        if keyword:
            forums = forums.filter(
                Q(title__icontains=keyword) | Q(short_description__icontains=keyword)
            )

        creator_profile = get_object_or_404(Profile, user_id = user)
        creator_full_name = get_object_or_404(CustomUser, id = user.id)
        profile_image_url = (
            creator_profile.profile_image.url if creator_profile and creator_profile.profile_image else None
        )

        serializer = ForumViewSerializer(forums, many=True)

        if not forums.exists():
            return Response(
                {"status": "error", "message": "No forums found created by you."},
                status=404
            )

        return Response({"status": "success", "forums": serializer.data
            #     "createdBy": {
            #     "profileImage": profile_image_url,
            #     "full_name": creator_full_name.full_name
            # },
            })

class ForumsJoinedByMeView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def get(self, request):
        user = request.user
        keyword = request.query_params.get("keyword", '')
        print(user.id)

 
        joined_forums = Forum.objects.filter(
            memberships__user=user,  
            is_deleted=False,
            permission_to_join="public",
        ).exclude(user=user).distinct()


        # Apply keyword search
        if keyword:
            joined_forums = joined_forums.filter(
                Q(title__icontains=keyword) | Q(short_description__icontains=keyword)
            )


        serializer = ForumViewSerializer(joined_forums, many=True)

        if not joined_forums.exists():
            return Response(
                {"status": "error", "message": "No forums found for the given categories."},
                status=404
            )

        return Response({"status": "success", "forums": serializer.data})

        
    
class ForumsByCategoriesView(APIView):
    permission_classes = [permissions.IsAuthenticated]

    def get(self, request):
        category_ids = request.query_params.getlist("category_ids[]")
        keyword = request.query_params.get("keyword", '')

        forums = Forum.objects.filter(
            permission_to_join="public", 
            is_deleted=False
        )

        if category_ids:
            forums = forums.filter(categories__id__in=category_ids).distinct()

        if keyword:
            forums = forums.filter(
                Q(title__icontains=keyword) | Q(short_description__icontains=keyword)
            )

        serializer = ForumViewSerializer(forums, many=True)

        if not forums.exists():
            return Response(
                {"status": "error", "message": "No forums found for the given categories."},
                status=404
            )

        return Response({"status": "success", "forums": serializer.data})

class QuestionCommentsView(APIView):
    def get(self, request, *args, **kwargs):
        question_id = request.query_params.get("question_id")

        if not question_id:
            return Response(
                {"success": False, "message": "question_id is required."},
                status=status.HTTP_400_BAD_REQUEST,
            )

        question = get_object_or_404(ForumQuestion, id=question_id, is_deleted=False)

        comments = Comment.objects.filter(question=question, is_deleted=False).order_by("-created_at")

        comments_data = [
            {
                "commentId": comment.id,
                "content": comment.comment,
                # "createdBy": {
                "username": comment.user.username,
                "profileImage": comment.user.profile.profile_image.url if comment.user.profile.profile_image else None,
                "full_name": comment.user.full_name,
                # },
                "createdDate": comment.created_at,
            }
            for comment in comments
        ]

        # Construct response
        response_data = {
            "questionId": question.id,
            "question": question.question,
            "content": question.content,
            "tags": question.tags,
            # "createdBy": {
            "username": question.user.username,
            "profileImage": question.user.profile.profile_image.url if question.user.profile.profile_image else None,
            "full_name": question.user.full_name,
            # },
            "createdDate": question.created_at,
            "comments": comments_data,
        }

        return Response(response_data, status=status.HTTP_200_OK)
    
class TrendingTopicsAPI(APIView):
    permission_classes = [permissions.IsAuthenticated]  # Allow public access for viewing

    def get(self, request):
        user = request.user
        print(user.id)
        trending_questions = (
            ForumQuestion.objects
            .filter(forum__permission_to_join="public", is_deleted=False)
            .annotate(
                total_comments=Count("question_comments", distinct=True),
                total_likes=Count("question_likes", distinct=True),
                user_liked=Exists(Like.objects.filter(question=OuterRef('id'), user=user))
            )
            .order_by("-total_comments", "-total_likes", "-created_at")  # Trending based on comments and likes
        )

        questions_data = [
            {
                "id": question.id,
                "forum_id": question.forum.id,
                "forum_name": question.forum.title,
                "user_id": question.user.id,
                "full_name": question.user.full_name,
                "profile_image": question.user.profile.profile_image.url if hasattr(question.user, 'profile') else None,
                "question": question.question,
                "content": question.content,
                "tags": question.tags,
                "created_at": question.created_at,
                "total_comments": question.total_comments,
                "total_likes": question.total_likes,
                "is_liked_by_user": question.user_liked
            }
            for question in trending_questions
        ]

        return Response({"trending_questions": questions_data}, status=200)